import 'dart:async';
import 'dart:convert';
import 'dart:ui' as ui;

import 'package:bakalarka_frontend/bloc/eraser_cubit.dart';
import 'package:bakalarka_frontend/bloc/offset_with_stroke_cubit.dart';
import 'package:bakalarka_frontend/bloc/stroke_cubit.dart';
import 'package:bakalarka_frontend/widgets/custom_canvas.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

class Annotation extends StatelessWidget {
  final String _image;

  Annotation(this._image);

  @override
  Widget build(BuildContext context) {
    var currentImage = Image.memory(
      base64Decode(_image),
    );
    return FutureBuilder(
        future: widthHeight(currentImage),
        builder: (context, snapshot) {
          if (!snapshot.hasData) return CircularProgressIndicator();
          List<int>? dimensions = snapshot.data as List<int>?;
          double widthRatio = ((MediaQuery.of(context).size.width - 50) * 0.8) /
              dimensions![0].toDouble();
          double heightRatio =
              ((MediaQuery.of(context).size.height - 50) * 0.8) /
                  dimensions[1].toDouble();
          double multiplier =
              widthRatio < heightRatio ? widthRatio : heightRatio;
          return Padding(
            padding: const EdgeInsets.all(25.0),
            child: Row(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                SizedBox(
                  width: (MediaQuery.of(context).size.width - 50) * 0.2,
                  height: (MediaQuery.of(context).size.height - 50) * 0.8,
                  child: ListView(
                    shrinkWrap: true,
                    children: [
                      Center(child: Text('Stroke')),
                      SizedBox(
                        height: 5,
                      ),
                      BlocBuilder<StrokeCubit, double>(
                          builder: (context, state) {
                        return Slider(
                          value: state,
                          onChanged: (value) {
                            BlocProvider.of<StrokeCubit>(context).update(value);
                          },
                          min: 1,
                          max: 70,
                          divisions: 69,
                          label: state.round().toString(),
                          inactiveColor: Colors.white,
                          activeColor: Colors.amber.shade100,
                          thumbColor: Colors.grey,
                        );
                      }),
                      SizedBox(
                        height: 10,
                      ),
                      TextButton(
                        onPressed: () {
                          BlocProvider.of<OffsetWithStrokeCubit>(context)
                              .undo();
                        },
                        child: Icon(Icons.undo),
                      ),
                      SizedBox(
                        height: 10,
                      ),
                      BlocBuilder<EraserCubit, bool>(builder: (context, state) {
                        return TextButton(
                          onPressed: () {
                            BlocProvider.of<EraserCubit>(context).change();
                          },
                          child: Icon(state
                              ? CupertinoIcons.paintbrush
                              : CupertinoIcons.paintbrush_fill),
                        );
                      }),
                      SizedBox(
                        height: 10,
                      ),
                      TextButton(
                        onPressed: () {
                          if (BlocProvider.of<OffsetWithStrokeCubit>(context)
                              .state
                              .last
                              .isNotEmpty)
                            BlocProvider.of<OffsetWithStrokeCubit>(context)
                                .emptyVersion();
                        },
                        child: Text('Delete'),
                      )
                    ],
                  ),
                ),
                SizedBox(
                  width: ((MediaQuery.of(context).size.width - 50) * 0.8 -
                          dimensions[0].toDouble() * multiplier) /
                      2,
                ),
                InteractiveViewer(
                  child: Container(
                    width: dimensions[0].toDouble() * multiplier,
                    height: dimensions[1].toDouble() * multiplier,
                    decoration: BoxDecoration(
                      image: DecorationImage(
                        fit: BoxFit.contain,
                        image: currentImage.image,
                      ),
                    ),
                    child: CustomCanvas(dimensions[0].toDouble() * multiplier,
                        dimensions[1].toDouble() * multiplier),
                  ),
                ),
              ],
            ),
          );
        });
  }
}

Future<List<int>> widthHeight(Image image) async {
  Completer<ui.Image> completer = new Completer<ui.Image>();
  image.image
      .resolve(new ImageConfiguration())
      .addListener(new ImageStreamListener((ImageInfo image, bool _) {
    completer.complete(image.image);
  }));
  ui.Image info = await completer.future;
  return [info.width, info.height];
}
