import 'dart:collection';
import 'dart:ui';

import 'package:bakalarka_frontend/bloc/eraser_cubit.dart';
import 'package:bakalarka_frontend/bloc/offset_with_stroke_cubit.dart';
import 'package:bakalarka_frontend/bloc/stroke_cubit.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

class CustomCanvas extends StatelessWidget {
  final double _width, _height;

  const CustomCanvas(this._width, this._height);

  @override
  Widget build(BuildContext context) {
    return ClipRect(
      child: GestureDetector(
        behavior: HitTestBehavior.opaque,
        onPanDown: (details) {
          BlocProvider.of<OffsetWithStrokeCubit>(context).newVersion();
          BlocProvider.of<OffsetWithStrokeCubit>(context).addToVersion(null);
          BlocProvider.of<OffsetWithStrokeCubit>(context).addToVersion(
              OffsetWithStroke(
                  details.localPosition,
                  BlocProvider.of<StrokeCubit>(context).state,
                  BlocProvider.of<EraserCubit>(context).state));
        },
        onPanUpdate: (details) {
          if (details.localPosition.dy >= 0 &&
              details.localPosition.dx >= 0 &&
              details.localPosition.dy < _height &&
              details.localPosition.dx < _width)
            BlocProvider.of<OffsetWithStrokeCubit>(context).addToVersion(
                OffsetWithStroke(
                    details.localPosition,
                    BlocProvider.of<StrokeCubit>(context).state,
                    BlocProvider.of<EraserCubit>(context).state));
          else if (BlocProvider.of<OffsetWithStrokeCubit>(context)
                  .state
                  .last
                  .isNotEmpty &&
              BlocProvider.of<OffsetWithStrokeCubit>(context).state.last.last !=
                  null)
            BlocProvider.of<OffsetWithStrokeCubit>(context).addToVersion(null);
        },
        onPanEnd: (details) {
          BlocProvider.of<OffsetWithStrokeCubit>(context).addToVersion(null);
        },
        child: Opacity(
          opacity: 0.5,
          child: BlocBuilder<OffsetWithStrokeCubit,
              UnmodifiableListView<UnmodifiableListView<OffsetWithStroke?>>>(
            builder: (context, state) {
              if (state.isEmpty) return Container();
              return CustomPaint(
                painter: CustomCanvasPainter(state.last),
              );
            },
          ),
        ),
      ),
    );
  }
}

class CustomCanvasPainter extends CustomPainter {
  final List<OffsetWithStroke?> points;

  CustomCanvasPainter(this.points);

  @override
  void paint(Canvas canvas, Size size) {
    Paint brush = Paint();
    brush.color = Colors.red;
    brush.isAntiAlias = true;

    for (int i = 0; i < points.length - 1; i++) {
      if (points[i] != null && points[i + 1] != null) {
        brush.strokeWidth = points[i]!.stroke;
        brush.blendMode =
            points[i]!.isEraser ? BlendMode.clear : BlendMode.color;
        brush.strokeCap = points[i]!.cap;
        canvas.drawLine(points[i]!.offset, points[i + 1]!.offset, brush);
      } else if (points[i] != null) {
        brush.strokeWidth = points[i]!.stroke;
        brush.blendMode =
            points[i]!.isEraser ? BlendMode.clear : BlendMode.color;
        brush.strokeCap = points[i]!.cap;
        canvas.drawPoints(PointMode.points, [points[i]!.offset], brush);
      }
    }
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}
