import 'package:bakalarka_frontend/pages/overview.dart';
import 'package:file_picker_cross/file_picker_cross.dart';
import 'package:flutter/material.dart';
import 'package:http/http.dart';
import 'package:localstorage/localstorage.dart';

class DatasetCreationDialog extends StatefulWidget {
  DatasetCreationDialog({Key? key}) : super(key: key);

  @override
  _DatasetCreationDialogState createState() => _DatasetCreationDialogState();
}

class _DatasetCreationDialogState extends State<DatasetCreationDialog> {
  final LocalStorage _storage = LocalStorage('info');
  List<FilePickerCross> _images = [];
  var _textControler = TextEditingController();

  @override
  Widget build(BuildContext context) {
    Map<String, dynamic> datasets = _storage.getItem('datasets') == null
        ? Map()
        : _storage.getItem('datasets');
    return AlertDialog(
      title: Text('Import dataset'),
      content: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          TextField(
            controller: _textControler,
            decoration: InputDecoration(hintText: 'Name dataset'),
          ),
          SizedBox(
            height: 30,
          ),
          Text(_images.length.toString() + ' images selected'),
        ],
      ),
      actions: [
        TextButton(
          onPressed: () async {
            var temp = await FilePickerCross.importMultipleFromStorage(
                type: FileTypeCross.image, fileExtension: 'jpg, jpeg, png');
            setState(() {
              _images = temp;
            });
          },
          child: Text('Choose files'),
        ),
        TextButton(
          onPressed: () async {
            if (_images.isEmpty) {
              ScaffoldMessenger.of(context).showSnackBar(SnackBar(
                content: Text('No images were chosen'),
                duration: Duration(milliseconds: 400),
                behavior: SnackBarBehavior.fixed,
                padding: EdgeInsets.all(5),
                elevation: 500,
              ));
              return;
            }
            var client = Client();
            try {
              for (var image in _images) {
                client
                    .post(Uri.parse('http://localhost:8000/upload.php'), body: {
                  'dataset': _textControler.text,
                  'name': image.fileName,
                  'image': image.toBase64(),
                }, headers: {
                  "Access-Control-Allow-Origin": "*"
                });
              }
              datasets[_textControler.text] = Map();
              datasets[_textControler.text]!['images'] =
                  _images.map((image) => image.fileName).toList();
              datasets[_textControler.text]['parameters'] = Map();
              datasets[_textControler.text]['parameters']['epochs'] = 10;
              datasets[_textControler.text]['parameters']['validation'] = 0.2;
              datasets[_textControler.text]['parameters']['batch'] = 20;
              datasets[_textControler.text]['parameters']['patch'] = 200;
              datasets[_textControler.text]['parameters']['overlap'] = 50;
              _storage.setItem('datasets', datasets);
              Navigator.of(context).pop();
              Navigator.of(context).pop();
              Navigator.of(context)
                  .push(MaterialPageRoute(builder: (context) => Overview()));
              ScaffoldMessenger.of(context).showSnackBar(
                  SnackBar(content: Text('Dataset created successfully')));
            } finally {
              client.close();
            }
          },
          child: Text('Create'),
        ),
      ],
    );
  }
}
