import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:localstorage/localstorage.dart';

class HyperparametersDialog extends StatelessWidget {
  final LocalStorage _storage = LocalStorage('info');
  final TextEditingController _epochs = TextEditingController();
  final TextEditingController _validation = TextEditingController();
  final TextEditingController _patch = TextEditingController();
  final TextEditingController _batch = TextEditingController();
  final TextEditingController _overlap = TextEditingController();
  final String _datasetName;

  HyperparametersDialog(this._datasetName);

  @override
  Widget build(BuildContext context) {
    Map<String, dynamic> datasets = _storage.getItem('datasets') == null
        ? Map()
        : _storage.getItem('datasets');
    if (datasets[_datasetName]['parameters'] == null)
      datasets[_datasetName]['parameters'] = Map();
    return AlertDialog(
      title: Text('Learning and preprocessing hyperparameters'),
      content: Column(
        children: [
          numberInput(_epochs, 'Number of epochs',
              datasets[_datasetName]['parameters']['epochs']),
          numberInput(_validation, 'Ratio of images used for validation (%)',
              datasets[_datasetName]['parameters']['validation'] * 100),
          numberInput(_batch, 'Batch size',
              datasets[_datasetName]['parameters']['batch']),
          numberInput(_patch, 'Patch size',
              datasets[_datasetName]['parameters']['patch']),
          numberInput(_overlap, 'Overlap of patches (%)',
              datasets[_datasetName]['parameters']['overlap'])
        ],
      ),
      actions: [
        TextButton(
          onPressed: () {
            datasets[_datasetName]['parameters']['epochs'] =
                int.parse(_epochs.text);
            datasets[_datasetName]['parameters']['validation'] =
                double.parse(_validation.text) / 100;
            datasets[_datasetName]['parameters']['batch'] =
                int.parse(_batch.text);
            datasets[_datasetName]['parameters']['patch'] =
                int.parse(_patch.text);
            datasets[_datasetName]['parameters']['overlap'] =
                int.parse(_overlap.text);
            _storage.setItem('datasets', datasets);
            Navigator.of(context).pop();
          },
          child: Text('Save'),
        )
      ],
    );
  }

  TextField numberInput(controller, text, initial) {
    controller.text = initial.toString();
    return TextField(
      controller: controller,
      keyboardType: TextInputType.number,
      inputFormatters: [FilteringTextInputFormatter.digitsOnly],
      decoration: InputDecoration(
        labelText: text,
      ),
    );
  }
}
