import 'package:bakalarka_frontend/pages/iteration.dart';
import 'package:flutter/material.dart';
import 'package:localstorage/localstorage.dart';

class NewIterationDialog extends StatelessWidget {
  final LocalStorage _storage = LocalStorage('info');
  final String _datasetName;

  NewIterationDialog(this._datasetName);

  @override
  Widget build(BuildContext context) {
    Map<String, dynamic> datasets = _storage.getItem('datasets') == null
        ? Map()
        : _storage.getItem('datasets');
    Map<String, dynamic> iterations = _storage.getItem('iterations') == null
        ? Map()
        : _storage.getItem('iterations');
    var _manualController = TextEditingController(
        text: datasets[_datasetName]['preannotated'] != null
            ? datasets[_datasetName]['preannotated'].length.toString()
            : '');
    var _preannotationController = TextEditingController();
    return AlertDialog(
      title: Text('Start new iteration'),
      content: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          TextField(
            controller: _manualController,
            decoration: InputDecoration(
                hintText: 'Number of images to be annotated manually'),
          ),
          SizedBox(
            height: 10,
          ),
          TextField(
            controller: _preannotationController,
            decoration: InputDecoration(
                hintText: 'Number of images to be pre-annotated'),
          )
        ],
      ),
      actions: [
        TextButton(
          onPressed: () {
            if (_preannotationController.text == '') {
              showError(
                  context, 'Number of annotated images is a required field');
              return;
            }
            if (_manualController.text == '') {
              showError(context,
                  'Number of pre-annotated images is a required field');
              return;
            }
            if (datasets[_datasetName]['images'].length -
                        (datasets[_datasetName]!.containsKey('annotated')
                            ? datasets[_datasetName]!['annotated'].length
                            : 0) -
                        (datasets[_datasetName]!.containsKey('preannotated')
                            ? datasets[_datasetName]!['preannotated'].length
                            : 0) <
                    int.parse(_preannotationController.text) ||
                datasets[_datasetName]['images'].length -
                        (datasets[_datasetName]!.containsKey('annotated')
                            ? datasets[_datasetName]!['annotated'].length
                            : 0) <
                    int.parse(_manualController.text) +
                        int.parse(_preannotationController.text)) {
              showError(context,
                  'The number of annotated and pre-annotated images cannot exceed total number of images');
              return;
            }
            if (int.parse(_manualController.text) <= 0) {
              showError(context,
                  'The number of annotated images needs to be positive');
              return;
            }
            if (int.parse(_preannotationController.text) < 0) {
              showError(context,
                  'The number of pre-annotated images needs to be positive');
              return;
            }
            iterations[_datasetName] = Map<String, dynamic>();
            iterations[_datasetName]['manual_annotation'] =
                int.parse(_manualController.text);
            iterations[_datasetName]['model_preannotation'] =
                int.parse(_preannotationController.text);
            _storage.setItem('iterations', iterations);
            Navigator.of(context).pop();
            Navigator.of(context).push(MaterialPageRoute(
                builder: (context) => Iteration(_datasetName)));
          },
          child: Text('Create'),
        )
      ],
    );
  }

  void showError(BuildContext context, String message) {
    showDialog(
      context: context,
      builder: (context) => Dialog(
        child: Padding(
          padding: const EdgeInsets.all(5.0),
          child: Stack(
            children: [
              Container(
                padding: EdgeInsets.symmetric(vertical: 30, horizontal: 50),
                width: MediaQuery.of(context).size.width / 2.5,
                child: Text(message),
              ),
              Positioned(
                child: InkWell(
                  onTap: () {
                    Navigator.of(context, rootNavigator: true).pop();
                  },
                  child: Icon(
                    Icons.close,
                    color: Colors.grey,
                  ),
                ),
                right: 0,
                top: 0,
              ),
            ],
          ),
        ),
      ),
      barrierDismissible: true,
    );
  }
}
