import 'dart:convert';

import 'package:bakalarka_frontend/pages/dataset_detail.dart';
import 'package:bakalarka_frontend/pages/iteration.dart';
import 'package:bakalarka_frontend/widgets/training_progress_dialog.dart';
import 'package:flutter/material.dart';
import 'package:http/http.dart';
import 'package:localstorage/localstorage.dart';

class Training extends StatelessWidget {
  final LocalStorage _storage = LocalStorage('info');
  final String _datasetName;

  Training(this._datasetName);

  @override
  Widget build(BuildContext context) {
    Map<String, dynamic> datasets = _storage.getItem('datasets') == null
        ? Map()
        : jsonDecode(jsonEncode(_storage.getItem('datasets')));
    Map<String, dynamic> iterations = _storage.getItem('iterations') == null
        ? Map()
        : jsonDecode(jsonEncode(_storage.getItem('iterations')));
    return Center(
      child: TextButton(
        child: Text('Train NN'),
        onPressed: () async {
          List<List<String>> inputs =
              await prepareInputs(datasets, 'annotated');
          List<String> masks = await prepareMasks(datasets);
          List<String> infer = [];
          for (var i in datasets[_datasetName]['images']) {
            if (iterations[_datasetName]['model_preannotation'] == 0) break;
            if (!datasets[_datasetName]['annotated'].contains(i) &&
                (datasets[_datasetName]['preannotated'] == null ||
                    !datasets[_datasetName]['preannotated'].contains(i))) {
              iterations[_datasetName]['model_preannotation'] -= 1;
              infer.add(i);
            }
          }
          if (datasets[_datasetName]['preannotated'] == null)
            datasets[_datasetName]['preannotated'] = [];
          datasets[_datasetName]['preannotated'].addAll(infer);
          Map<String, Map<String, dynamic>> tempDatasets = Map();
          tempDatasets[_datasetName] = Map();
          tempDatasets[_datasetName]!['images'] = infer;
          List<List<String>> inferr =
              await prepareInputs(tempDatasets, 'images');
          List<String> inferred = inferr
              .map((name) => name[0]
                  .replaceAll('.jpg', '.png')
                  .replaceAll(_datasetName + '/', _datasetName + '/.premask'))
              .toList();
          showDialog(
            context: context,
            builder: (context) =>
                TrainingProgressDialog(datasetId: _datasetName),
            barrierDismissible: false,
          );
          post(
            Uri.parse('http://0.0.0.0:5000/'),
            body: jsonEncode({
              'dataset_id': _datasetName,
              'train': {
                'inputs': inputs,
                'gts': masks,
                'validation_ratio': datasets[_datasetName]['parameters']
                    ['validation'],
                'epoch_count': datasets[_datasetName]['parameters']['epochs'],
              },
              'preprocessing_params': {
                'patch_size': datasets[_datasetName]['parameters']['patch'],
                'batch_size': datasets[_datasetName]['parameters']['batch'],
                'overlap': datasets[_datasetName]['parameters']['overlap'],
              },
              'inference': {
                'inputs': inferr,
                'outputs': inferred,
              },
            }),
            headers: {
              'Content-Type': 'application/json; charset=UTF-8',
              'Access-Control-Allow-Origin': '*',
            },
          ).then((response) {
            if (response.statusCode == 200) {
              iterations.remove(_datasetName);
              _storage.setItem('iterations', iterations);
              _storage.setItem('datasets', datasets);
              Navigator.of(context).pop();
              Navigator.of(context).pop();
              Navigator.of(context).pop();
              Navigator.of(context).push(MaterialPageRoute(
                  builder: (context) => DatasetDetail(_datasetName)));
            } else {
              showDialog(
                context: context,
                builder: (context) => AlertDialog(
                  title: Text('An error occured during training or inference'),
                  content: Text(response.body),
                  actions: [
                    TextButton(
                      onPressed: () {
                        Navigator.of(context).pop();
                        Navigator.of(context).pop();
                        Navigator.of(context).pop();
                        Navigator.of(context).push(MaterialPageRoute(
                            builder: (context) => new Iteration(_datasetName)));
                      },
                      child: Text('Close'),
                    )
                  ],
                ),
                barrierDismissible: false,
              );
            }
          });
        },
      ),
    );
  }

  Future<List<String>> prepareMasks(Map<String, dynamic> datasets) async {
    return prepareFields(
        datasets, 'annotated', (name) => '.mask' + name.split('.')[0] + '.png');
  }

  Future<List<List<String>>> prepareInputs(
      Map<String, dynamic> datasets, String identifier) async {
    return (await prepareFields(datasets, identifier, (name) => name))
        .map((image) => [image])
        .toList();
  }

  Future<List<String>> prepareFields(Map<String, dynamic> datasets,
      String identifier, String Function(String) modifier) async {
    List<String> images = [];
    var client = Client();
    for (var i in datasets[_datasetName][identifier]) {
      var response = await client
          .post(Uri.parse('http://localhost:8000/abspath.php'), body: {
        'dataset': _datasetName,
        'name': modifier(i),
      }, headers: {
        "Access-Control-Allow-Origin": "*"
      });
      images.add(response.body);
    }
    return images;
  }
}
