import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:socket_io_client/socket_io_client.dart';

class TrainingProgressDialog extends StatefulWidget {
  final String datasetId;

  const TrainingProgressDialog({Key? key, required this.datasetId})
      : super(key: key);

  @override
  State<TrainingProgressDialog> createState() => _TrainingProgressDialogState();
}

class _TrainingProgressDialogState extends State<TrainingProgressDialog> {
  int _progress = 0;
  late final Socket socket;

  @override
  void dispose() {
    socket.disconnect();
    super.dispose();
  }

  @override
  void initState() {
    var message = jsonEncode({'dataset_id': widget.datasetId});
    socket = io(
        'ws://0.0.0.0:5001/',
        OptionBuilder().disableAutoConnect().setExtraHeaders({
          'Access-Control-Allow-Origin': '*',
          'message': message,
        }).build());
    socket.connect();
    socket.onConnect((_) {
      print('connected');
    });
    socket.on('report', (data) {
      if (data['progress'] == 100) {
        socket.disconnect();
      }
      this.setState(() {
        _progress = data['progress'];
      });
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return AlertDialog(
        title: Text('Training in progress'),
        content: SizedBox(
          width: 100,
          height: 100,
          child: Stack(children: [
            Center(
              child: SizedBox(
                width: 100,
                height: 100,
                child: CircularProgressIndicator(
                  value: _progress.toDouble() / 100,
                  backgroundColor: Colors.amberAccent.shade100,
                  color: Colors.lightGreenAccent,
                ),
              ),
            ),
            Center(child: Text(_progress.toString() + '%')),
          ]),
        ));
  }
}
